from birdseyelib.socketClient import Client
from birdseyelib.memory import Memory
from birdseyelib.controllerInput import ControllerInput
from birdseyelib.emulation import Emulation
from birdseyelib.externalTool import ExternalTool



class ExternalTool:
    """Class containing various functions for controlling the external tool."""
    def __init__(self, client) -> None:
        self.client = client

    def set_commandeer(self, enabled):
        """Sets the communication mode of the external tool to either manual or commandeer.

        :param enabled: Determines whether a request to enable commandeer or disable it should be sent. \
        `True` = enable commandeer, `False` = disable.
        :type enabled: bool"""
        self.client._queue_request("COMMANDEER;" + str(enabled) + "\n")



class ControllerInput:
    """Class containing various functions for reading memory from BizHawk."""
    def __init__(self, client) -> None:
        self.client = client

    def set_controller_input(self, a=False, b=False, up=False, down=False, right=False, left=False):
        """Sets the controller inputs to be executed in the emulator.
        All inputs are set to `False` be default.
        The inputs are executed until a new controller input is sent.

        :param a: The state of the A button.
        :type a: bool

        :param b: The state of the B button.
        :type b: bool

        :param up: The state of the Up button on the control pad.
        :type up: bool

        :param down: The state of the Down button on the control pad.
        :type down: bool

        :param right: The state of the Right button on the control pad.
        :type right: bool

        :param left: The state of the Left button on the control pad.
        :type left: bool"""
        bool_to_string = {False : "false", True : "true"}
        controller_input = bool_to_string[a] + ";" + bool_to_string[b] + ";" + \
                           bool_to_string[up] + ";" + bool_to_string[down] + ";" + \
                           bool_to_string[right] + ";" + bool_to_string[left]
        self.client._queue_request("INPUT;" + controller_input + "\n")



from setuptools import setup

from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="birds-eye-lib",
    version="1.0.1",
    description="A BizHawk external tool + python library for python scripts to communicate with the BizHawk emulator.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://birds-eye.readthedocs.io/",
    author="SkiHatDuckie",
    author_email="SkiHatDuckie@gmail.com",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent"
    ],
    packages=["birdseyelib"],
    include_package_data=True,
)


class Emulation:
    """Class containing various functions for controlling emulation in BizHawk."""
    def __init__(self, client) -> None:
        self.client = client

    def request_framecount(self):
        """Requests for the current framecount from the external tool."""
        self.client._queue_request("FRAME;\n")

    def get_framecount(self) -> int:
        """Returns the current framecount from the emulator, or `-1`, if no
        data has been received yet."""
        data = self.client._get_latest_response_data("FRAME")
        if data:
            return int(data)
        else:
            return -1
